import numpy as np

ALTITUDE_KM = 350
ALTITUDE_M = 1_000 * ALTITUDE_KM
RESOLUTION = 1  # Grid resolution in degrees
LONGITUDES = np.arange(-180, 180 + RESOLUTION, RESOLUTION)
LATITUDES = np.arange(-90, 90 + RESOLUTION, RESOLUTION)
